@@ -18,6 +18,8 @@
 #include <gdk-pixbuf/gdk-pixbuf.h>
 #include <libart_lgpl/libart.h>
 
+#include "egg-screen-help.h"
+
 #define FISH_PREFS_NAME		"name"
 #define FISH_PREFS_IMAGE	"image"
 #define FISH_PREFS_COMMAND	"command"
@@ -53,7 +55,12 @@ typedef struct {
 	PanelAppletOrient  orient;
 } Fish;
 
-GtkWidget *bah_window = NULL;
+static inline GdkWindow *
+fishy_root_window (Fish *fish)
+{
+	return gdk_screen_get_root_window (
+			gtk_widget_get_screen (fish->applet));
+}
 
 /*
  * set_access_name_desc
@@ -171,44 +178,43 @@ fish_applet_rotate (Fish *fish)
 	return retval;
 }
 
-static gchar *
-get_location(void)
+static char *
+get_location (void)
 {
-	static gchar location[256];
-	FILE *zone;
+	static char  location [256];
+	char         buffer [256];
+	FILE        *zone;
+	int          i, len, count;
 	
-	/* Now we try old method. Works for glibc < 2.2 */
+	/* Old method : works for glibc < 2.2 */
 	zone = fopen("/etc/timezone", "r");
-	if (zone != NULL) {
-		fscanf(zone, "%255s", location);
-		fclose(zone);
+	if (zone) {
+		fscanf (zone, "%255s", location);
+		fclose (zone);
 		return location;
-	} else { /* We try new method for glibc 2.2 */
-		gchar buf[256];
-		int i, len, count;
-		
-		len = readlink("/etc/localtime", buf, sizeof(buf));
-		if (len <= 0)
-			return NULL;
+	} 
 
-		for (i = len, count = 0; (i > 0) && (count != 2); i--)
-			if (buf[i] == '/')
-				count++;
+	/* New method : works for glibc 2.2 */
+	len = readlink ("/etc/localtime", buffer, sizeof (buffer));
+	if (len <= 0)
+		return NULL;
 
-		if (count != 2)
-			return NULL;
+	for (i = len, count = 0; (i > 0) && (count != 2); i--)
+		if (buffer [i] == '/')
+			count++;
 
-		memcpy(location, &buf[i + 2], len - i - 2);
-		return location;
-	}
-	
-	return NULL;
+	if (count != 2)
+		return NULL;
+
+	memcpy (location, &buffer [i + 2], len - i - 2);
+
+	return location;
 }
 
 static void 
-set_wanda_day(void)
+set_wanda_day (void)
 {
-	const gchar *spanish_timezones[] = {
+	const char *spanish_timezones [] = {
 		"Europe/Madrid",
 		"Africa/Ceuta",
 		"Atlantic/Canary",
@@ -218,14 +224,14 @@ set_wanda_day(void)
 		"Mexico/General",
 		NULL
 	};
-	gchar *location = get_location();
-	int i;
-	gboolean found = FALSE;
+	gboolean  found;
+	char     *location;
+	int       i;
 	
-	if (location == NULL) /* We couldn't guess our location */
+	if (!(location = get_location ()))
 		return;
 	
-	for (i = 0; !found && spanish_timezones[i]; i++)
+	for (i = 0, found = FALSE; !found && spanish_timezones[i]; i++)
 		if (!g_strcasecmp(spanish_timezones[i], location)) {
 			/* Hah!, We are in Spain ot Mexico */
 			/* Spanish fool's day: 28th December */
@@ -252,9 +258,9 @@ load_image_file (Fish *fish)
 	if (fish->pix)
 		gdk_pixmap_unref (fish->pix);
 
-	g_assert (bah_window && bah_window->window);
-
 	frames = panel_applet_gconf_get_int (PANEL_APPLET (fish->applet), FISH_PREFS_FRAMES, NULL);
+	if (frames <= 0)
+		frames = 1;
 
 	tmp = panel_applet_gconf_get_string (PANEL_APPLET (fish->applet), FISH_PREFS_IMAGE, NULL);
 	if (g_path_is_absolute (tmp)) {
@@ -285,10 +291,8 @@ load_image_file (Fish *fish)
 			fish->w = frames * fish->size;
 			fish->h = fish->size;
 		}
-
- 		fish->pix = gdk_pixmap_new (bah_window->window,
+ 		fish->pix = gdk_pixmap_new (fishy_root_window (fish),
 					    fish->w, fish->h, -1);
-
 		return;
 	}
 
@@ -353,7 +357,7 @@ load_image_file (Fish *fish)
 	fish->w = w;
 	fish->h = h;
 
-	fish->pix = gdk_pixmap_new (bah_window->window,
+	fish->pix = gdk_pixmap_new (fishy_root_window (fish),
 				    fish->w,fish->h,-1);
 
 	gc = gdk_gc_new (fish->pix);
@@ -381,6 +385,8 @@ setup_size (Fish *fish)
 	gint frames;
 
 	frames = panel_applet_gconf_get_int (PANEL_APPLET (fish->applet), FISH_PREFS_FRAMES, NULL);
+	if (frames <= 0)
+		frames = 1;
 
 	if (fish_applet_rotate (fish)) {
 		GTK_WIDGET (fish->darea)->requisition.width = fish->w;
@@ -417,6 +423,8 @@ fish_draw (GtkWidget *darea,
 		return;
 
 	frames = panel_applet_gconf_get_int (PANEL_APPLET (fish->applet), FISH_PREFS_FRAMES, NULL);
+	if (frames <= 0)
+		frames = 1;
 
 	if (fish_applet_rotate (fish))
 		gdk_draw_pixmap (fish->darea->window,
@@ -466,6 +474,8 @@ fish_timeout(gpointer data)
 		gint frames;
 
 		frames = panel_applet_gconf_get_int (PANEL_APPLET (fish->applet), FISH_PREFS_FRAMES, NULL);
+		if (frames <= 0)
+			frames = 1;
 
 		fish->curpix++;
 		if (fish->curpix >= frames)
@@ -506,7 +516,7 @@ apply_dialog_properties (Fish *fish)
 static void
 apply_properties (Fish *fish) 
 {
-	gdouble     speed;
+	gdouble speed;
 
 	apply_dialog_properties (fish);
 	
@@ -518,6 +528,8 @@ apply_properties (Fish *fish)
 		gtk_timeout_remove (fish->timeout_id);
 
 	speed = panel_applet_gconf_get_float (PANEL_APPLET (fish->applet), FISH_PREFS_SPEED, NULL);
+	if (speed <= 0)
+		speed = 1.0;
 
         fish->timeout_id = gtk_timeout_add (speed * 1000, fish_timeout, fish);
 	fish->curpix = 0;
@@ -584,6 +596,8 @@ fish_properties_apply (GtkDialog *pb, Fish *fish)
 						      "which would make the applet "
 						      "\"practical\" or useful."));
 			gtk_window_set_wmclass (GTK_WINDOW (w), "fish_useful_warning", "Fish");
+			gtk_window_set_screen (GTK_WINDOW (w),
+					       gtk_widget_get_screen (fish->applet));
 
 			gtk_widget_show_all (w);
 
@@ -614,11 +628,12 @@ fish_properties_apply (GtkDialog *pb, Fish *fish)
 }
 
 static void
-phelp (void)
+phelp (Fish *fish)
 {
 	GError *error = NULL;
 
-	gnome_help_display_desktop (
+	egg_screen_help_display_desktop (
+		gtk_widget_get_screen (fish->applet),
 		NULL, "fish-applet-2", "fish-applet-2", NULL, &error);
 	if (error) {
 		g_warning ("help error: %s\n", error->message);
@@ -648,7 +663,7 @@ response_cb (GtkDialog *dialog, gint id, gpointer data)
 			fish_properties_apply (dialog, fish);
 			break;
 		case GTK_RESPONSE_HELP:
-			phelp ();
+			phelp (fish);
 			break;
 		case GTK_RESPONSE_CLOSE:
 			gtk_widget_destroy (GTK_WIDGET(dialog));
@@ -678,7 +693,9 @@ display_properties_dialog (BonoboUIComponent *uic,
 	gboolean       rotate;
 	GtkWidget     *apply_button;
 
-	if (fish->pb != NULL) {
+	if (fish->pb) {
+		gtk_window_set_screen (GTK_WINDOW (fish->pb),
+				       gtk_widget_get_screen (fish->applet));
 		gtk_window_present (GTK_WINDOW (fish->pb));
 		return;
 	}
@@ -702,6 +719,8 @@ display_properties_dialog (BonoboUIComponent *uic,
 				GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE, 
 				GTK_STOCK_OK, GTK_RESPONSE_OK, NULL);
 	gtk_window_set_wmclass (GTK_WINDOW (fish->pb), "fish", "Fish");
+	gtk_window_set_screen (GTK_WINDOW (fish->pb),
+			       gtk_widget_get_screen (fish->applet));
 
 	gtk_dialog_set_default_response (GTK_DIALOG (fish->pb), GTK_RESPONSE_OK);
 
@@ -831,27 +850,54 @@ display_properties_dialog (BonoboUIComponent *uic,
 	g_free (command);
 }
 
+static void
+something_fishy_going_on (Fish       *fish,
+			  const char *message)
+{
+	GtkWidget *dialog;
+
+	dialog = gtk_message_dialog_new (NULL,
+					 GTK_DIALOG_DESTROY_WITH_PARENT,
+					 GTK_MESSAGE_ERROR,
+					 GTK_BUTTONS_CLOSE,
+					 message);
+
+	g_signal_connect (dialog, "response",
+			  G_CALLBACK (gtk_widget_destroy),
+			  NULL);
+
+	gtk_window_set_resizable (GTK_WINDOW (dialog), FALSE);
+	gtk_window_set_screen (GTK_WINDOW (dialog),
+			       gtk_widget_get_screen (fish->applet));
+	gtk_widget_show (dialog);
+}
+
 static gchar *
 fish_locate_fortune_command (Fish *fish)
 {
-	gchar *command;
-	gchar *tmp;
+	char *retval = NULL;
 
-	tmp = panel_applet_gconf_get_string (PANEL_APPLET (fish->applet), FISH_PREFS_COMMAND, NULL);
-	if (!tmp)
+	retval = panel_applet_gconf_get_string (PANEL_APPLET (fish->applet), FISH_PREFS_COMMAND, NULL);
+	if (!retval) {
+		something_fishy_going_on (fish, _("Unable to get the name of the command to execute"));
 		return NULL;
+	}
 
-	if (g_path_is_absolute (tmp))
-		return tmp;
+	if (!g_path_is_absolute (retval)) {
+		char *tmp;
 
-	command = g_find_program_in_path (tmp);
-	g_free (tmp);
+		tmp = retval;
+		retval = g_find_program_in_path (tmp);
+		g_free (tmp);
+	}
 
-	if (!command)
-		if (g_file_test ("/usr/games/fortune", G_FILE_TEST_EXISTS))
-			command = g_strdup ("/usr/games/fortune");
+	if (!retval && g_file_test ("/usr/games/fortune", G_FILE_TEST_EXISTS))
+		retval = g_strdup ("/usr/games/fortune");
 
-	return command;
+	if (!retval)
+		something_fishy_going_on (fish, _("Unable to locate the command to execute"));
+
+	return retval;
 }
 
 static void
@@ -896,8 +942,14 @@ delete_event (GtkWidget *w, gpointer data)
 static void 
 update_fortune_dialog (Fish *fish)
 {
-	char *fortune_command;
-	gchar *output = NULL;
+	GError *error = NULL;
+	char   *fortune_command;
+	char   *output = NULL;
+	char   *output_utf8;
+
+	fortune_command = fish_locate_fortune_command (fish);
+	if (!fortune_command)
+		return;
 
 	if ( fish->fortune_dialog == NULL ) {
 		GtkWidget *view;
@@ -918,6 +970,8 @@ update_fortune_dialog (Fish *fish)
 		g_signal_connect (G_OBJECT (fish->fortune_dialog), "response",
 				  G_CALLBACK (response), NULL);
 		gtk_window_set_wmclass (GTK_WINDOW (fish->fortune_dialog), "fish", "Fish");
+		gtk_window_set_screen (GTK_WINDOW (fish->fortune_dialog),
+				       gtk_widget_get_screen (fish->applet));
 		gnome_window_icon_set_from_file (GTK_WINDOW (fish->fortune_dialog),
 						 GNOME_ICONDIR"/gnome-fish.png");
 
@@ -959,19 +1013,35 @@ update_fortune_dialog (Fish *fish)
 		gtk_widget_show_all (fish->fortune_dialog);
 	} else {
 		set_access_name_desc (NULL, fish);
+		gtk_window_set_screen (GTK_WINDOW (fish->fortune_dialog),
+				       gtk_widget_get_screen (fish->applet));
 		gtk_window_present (GTK_WINDOW (fish->fortune_dialog));
 	}
 
 	text_clear (fish);
 
-	fortune_command = fish_locate_fortune_command (fish);
+	g_spawn_command_line_sync (fortune_command, &output, NULL, NULL, &error);
+	if (error) {
+		char *message;
 
-	g_spawn_command_line_sync (fortune_command, &output, NULL, NULL, NULL);
+		message = g_strdup_printf (_("Unable to execute '%s'\n\nDetails : %s"),
+					   fortune_command, error->message);
+		something_fishy_going_on (fish, message);
+		g_free (message);
+		g_error_free (error);
+	}
 	
 	g_free (fortune_command);
 
-	if (output) {
-		insert_text (fish, output);
+	/* The output is not guarantied to be in UTF-8 format, most
+	   likely it's just in ASCII-7 or in the user locale */
+	if (!g_utf8_validate (output, -1, NULL))
+	  output_utf8 = g_locale_to_utf8 (output, -1, NULL, NULL, NULL);
+	else
+	  output_utf8 = g_strdup (output);
+
+	if (output_utf8) {
+		insert_text (fish, output_utf8);
 	} else {
 		insert_text (fish, 
 			     _("You do not have fortune installed "
@@ -979,7 +1049,8 @@ update_fortune_dialog (Fish *fish)
 			       "to run.\n\nPlease refer to fish "
 			       "properties dialog."));
 	}
-
+	g_free (output);
+	g_free (output_utf8);
 }
 
 static void
@@ -991,6 +1062,8 @@ change_water (Fish *fish)
 				    _("The water needs changing!\n"
 				    "(Look at today's date)"));
 	gtk_window_set_wmclass (GTK_WINDOW (w), "fish", "Fish");
+	gtk_window_set_screen (GTK_WINDOW (w),
+			       gtk_widget_get_screen (fish->applet));
 
 	gtk_widget_show_all (w);
 
@@ -1050,9 +1123,11 @@ fish_expose (GtkWidget      *darea,
 	     GdkEventExpose *event,
 	     Fish           *fish)
 {
-	gint frames;
+	int frames;
 
 	frames = panel_applet_gconf_get_int (PANEL_APPLET (fish->applet), FISH_PREFS_FRAMES, NULL);
+	if (frames <= 0)
+		frames = 1;
 
 	if (fish_applet_rotate (fish))
 		gdk_draw_pixmap (fish->darea->window,
@@ -1083,7 +1158,11 @@ create_fish_widget(Fish *fish)
 	frames = panel_applet_gconf_get_int   (PANEL_APPLET (fish->applet), FISH_PREFS_FRAMES, NULL);
 	speed  = panel_applet_gconf_get_float (PANEL_APPLET (fish->applet), FISH_PREFS_SPEED, NULL);
 
-	gtk_widget_push_colormap (gdk_rgb_get_cmap ());
+	/* Sanity checking */
+	if (frames <= 0)
+		frames = 1;
+	if (speed <= 0)
+		speed = 1.0;
 
 	fish->darea = gtk_drawing_area_new();
 
@@ -1121,8 +1200,6 @@ create_fish_widget(Fish *fish)
         fish->timeout_id = gtk_timeout_add (speed * 1000, fish_timeout, fish);
 
         gtk_container_add(GTK_CONTAINER(fish->frame),fish->darea);
-        
-        gtk_widget_pop_colormap ();
 }
 
 static void
@@ -1132,7 +1209,8 @@ display_help_dialog (BonoboUIComponent *uic,
 {
 	GError *error = NULL;
 
-	gnome_help_display_desktop (
+	egg_screen_help_display_desktop (
+		gtk_widget_get_screen (fish->applet),
 		NULL, "fish-applet-2", "fish-applet-2", NULL, &error);
 	if (error) {
 		g_warning ("help error: %s\n", error->message);
@@ -1155,7 +1233,9 @@ display_about_dialog (BonoboUIComponent *uic,
 	gchar       *file;
 	gchar       *name;
 
-	if (fish->aboutbox != NULL) {
+	if (fish->aboutbox) {
+		gtk_window_set_screen (GTK_WINDOW (fish->aboutbox),
+				       gtk_widget_get_screen (fish->applet));
 		gtk_window_present (GTK_WINDOW (fish->aboutbox));
 		return;
 	}
@@ -1200,6 +1280,8 @@ display_about_dialog (BonoboUIComponent *uic,
 		gdk_pixbuf_unref (pixbuf);
 
 	gtk_window_set_wmclass (GTK_WINDOW (fish->aboutbox), "fish", "Fish");
+	gtk_window_set_screen (GTK_WINDOW (fish->aboutbox),
+			       gtk_widget_get_screen (fish->applet));
 	gnome_window_icon_set_from_file (GTK_WINDOW (fish->aboutbox),
 					 GNOME_ICONDIR"/gnome-fish.png");
 	gtk_signal_connect (GTK_OBJECT (fish->aboutbox), "destroy",
@@ -1318,17 +1400,6 @@ fish_applet_fill (PanelApplet *applet)
 
 	panel_applet_add_preferences (applet, "/schemas/apps/fish_applet/prefs", NULL);
 
-	gtk_widget_push_colormap (gdk_rgb_get_cmap ());
-
-	bah_window = gtk_window_new (GTK_WINDOW_POPUP);
-
-	gtk_widget_pop_colormap ();
-
-	gtk_widget_set_uposition (bah_window,
-				  gdk_screen_width () + 1,
-				  gdk_screen_height () + 1);
-	gtk_widget_show_now (bah_window);
-
 	fish->frame = gtk_frame_new (NULL);
 	gtk_frame_set_shadow_type (GTK_FRAME (fish->frame), GTK_SHADOW_IN);
 
