@@ -28,6 +28,8 @@
 #include "remote-helper.h"
 #include "mailcheck.h"
 
+#include "egg-screen-help.h"
+
 typedef enum {
 	MAILBOX_LOCAL,
 	MAILBOX_LOCALDIR,
@@ -35,8 +37,6 @@ typedef enum {
 	MAILBOX_IMAP
 } MailboxType;
 
-extern GtkTooltips *panel_tooltips;
-
 typedef struct _MailCheck MailCheck;
 struct _MailCheck {
 	char *mail_file;
@@ -161,7 +161,27 @@ static void set_atk_relation (GtkWidget *label, GtkWidget *entry, AtkRelationTyp
 #define WANT_BITMAPS(x) (x == REPORT_MAIL_USE_ANIMATION || x == REPORT_MAIL_USE_BITMAP)
 
 static void
-mailcheck_execute_shell (const char *command)
+set_tooltip (GtkWidget  *applet,
+	     const char *tip)
+{
+	GtkTooltips *tooltips;
+
+	tooltips = g_object_get_data (G_OBJECT (applet), "tooltips");
+	if (!tooltips) {
+		tooltips = gtk_tooltips_new ();
+		g_object_ref (tooltips);
+		gtk_object_sink (GTK_OBJECT (tooltips));
+		g_object_set_data_full (
+			G_OBJECT (applet), "tooltips", tooltips,
+			(GDestroyNotify) g_object_unref);
+	}
+
+	gtk_tooltips_set_tip (tooltips, applet, tip, NULL);
+}
+
+static void
+mailcheck_execute_shell (MailCheck  *mailcheck,
+			 const char *command)
 {
 	GError *error = NULL;
 
@@ -182,6 +202,8 @@ mailcheck_execute_shell (const char *command)
 				  NULL);
 
 		gtk_window_set_resizable (GTK_WINDOW (dialog), FALSE);
+		gtk_window_set_screen (GTK_WINDOW (dialog),
+				       gtk_widget_get_screen (GTK_WIDGET (mailcheck->applet)));
 
 		gtk_widget_show (dialog);
 
@@ -233,7 +255,7 @@ calc_dir_contents (char *dir)
 }
 
 static char *
-get_remote_password (void)
+get_remote_password (MailCheck *mc)
 {
 	GtkWidget *dialog;
 	GtkWidget *hbox;
@@ -270,6 +292,8 @@ get_remote_password (void)
 	gtk_widget_grab_focus (GTK_WIDGET (entry));
 
 	gtk_window_set_modal (GTK_WINDOW(dialog), TRUE);
+	gtk_window_set_screen (GTK_WINDOW (dialog),
+			       gtk_widget_get_screen (GTK_WIDGET (mc->applet)));
 
 	response = gtk_dialog_run (GTK_DIALOG (dialog));
 
@@ -362,7 +386,7 @@ check_mail_file_status (MailCheck *mc)
 				gtk_timeout_remove(mc->mail_timeout);
 				mc->mail_timeout = 0;
 			}
-			mc->real_password = get_remote_password();
+			mc->real_password = get_remote_password(mc);
 			mc->mail_timeout = gtk_timeout_add(mc->update_freq,
 							   mail_check_timeout,
 							   mc);
@@ -531,7 +555,7 @@ after_mail_check (MailCheck *mc)
 		if (mc->newmail_enabled &&
 		    mc->newmail_cmd && 
 		    (strlen(mc->newmail_cmd) > 0))
-			mailcheck_execute_shell (mc->newmail_cmd);
+			mailcheck_execute_shell (mc, mc->newmail_cmd);
 	}
 
 	switch (mc->report_mail_mode) {
@@ -577,9 +601,8 @@ after_mail_check (MailCheck *mc)
 		break;
 	}
 
-	gtk_tooltips_set_tip (panel_tooltips, GTK_WIDGET (mc->applet),
-			      text, NULL);
-	g_free(text);
+	set_tooltip (GTK_WIDGET (mc->applet), text);
+	g_free (text);
 }
 
 static gboolean
@@ -601,7 +624,7 @@ mail_check_timeout (gpointer data)
 			mc->mail_timeout = 0;
 		}
 
-		mailcheck_execute_shell (mc->pre_check_cmd);
+		mailcheck_execute_shell (mc, mc->pre_check_cmd);
 
 		mc->mail_timeout = gtk_timeout_add(mc->update_freq, mail_check_timeout, mc);
 	}
@@ -641,7 +664,7 @@ exec_clicked_cmd (GtkWidget *widget, GdkEventButton *event, gpointer data)
 	if (event->button == 1) {
 		
 		if (mc->clicked_enabled && mc->clicked_cmd && (strlen(mc->clicked_cmd) > 0))
-			mailcheck_execute_shell (mc->clicked_cmd);
+			mailcheck_execute_shell (mc, mc->clicked_cmd);
 		
 		if (mc->reset_on_clicked) {
 			mc->newmail = mc->unreadmail = 0;
@@ -1044,8 +1067,7 @@ set_mailbox_selection (GtkWidget *widget, gpointer data)
 		mc->remote_handle = NULL;
 	}
 	gtk_label_set_text (GTK_LABEL (mc->label), _("Status not updated"));
-	gtk_tooltips_set_tip (panel_tooltips, GTK_WIDGET (mc->applet),
-			      _("Status not updated"), NULL);
+	set_tooltip (GTK_WIDGET (mc->applet), _("Status not updated"));
 }
 
 static void
@@ -1554,7 +1576,7 @@ mailcheck_properties_page (MailCheck *mc)
 }
 
 static void
-phelp_cb (GtkDialog *w, gint tab, gpointer data)
+phelp_cb (GtkDialog *w, gint tab, MailCheck *mc)
 {
 	GError *error = NULL;
 	static GnomeProgram *applet_program = NULL;
@@ -1567,8 +1589,9 @@ phelp_cb (GtkDialog *w, gint tab, gpointer data)
      						      GNOME_PROGRAM_STANDARD_PROPERTIES, NULL);
 	}
 
-	gnome_help_display_desktop (applet_program, "mailcheck",
-				    "mailcheck","mailcheck-prefs", &error);
+	egg_screen_help_display_desktop (
+			gtk_widget_get_screen (GTK_WIDGET (mc->applet)),
+			applet_program, "mailcheck", "mailcheck","mailcheck-prefs", &error);
 	if (error) {
 		GtkWidget *dialog;
 		dialog = gtk_message_dialog_new (GTK_WINDOW (w),
@@ -1583,37 +1606,37 @@ phelp_cb (GtkDialog *w, gint tab, gpointer data)
 				  NULL);
 
 		gtk_window_set_resizable (GTK_WINDOW (dialog), FALSE);
+		gtk_window_set_screen (GTK_WINDOW (dialog),
+				       gtk_widget_get_screen (GTK_WIDGET (mc->applet)));
 		gtk_widget_show (dialog);
 		g_error_free (error);
 	}
 }	
 
 static void
-response_cb (GtkDialog *dialog, gint id, gpointer data)
+response_cb (GtkDialog *dialog, gint id, MailCheck *mc)
 {
-	MailCheck *mc = data;
-	if(id == GTK_RESPONSE_HELP)
-	{
-		phelp_cb (dialog,id,data);
+	if (id == GTK_RESPONSE_HELP) {
+		phelp_cb (dialog, id, mc);
 		return;	
 	}
+
 	gtk_widget_destroy (GTK_WIDGET (dialog));
 	mc->property_window = NULL;
-	
 }
 
 
 static void
-mailcheck_properties (BonoboUIComponent *uic, gpointer data, const gchar *verbname)
+mailcheck_properties (BonoboUIComponent *uic, MailCheck *mc, const gchar *verbname)
 {
 	GtkWidget *p;
 	GtkWidget *notebook;
 
-	MailCheck *mc = data;
-
-	if (mc->property_window != NULL) {
+	if (mc->property_window) {
+		gtk_window_set_screen (GTK_WINDOW (mc->property_window),
+				       gtk_widget_get_screen (GTK_WIDGET (mc->applet)));
 		gtk_window_present (GTK_WINDOW (mc->property_window));
-		return; /* Only one instance of the properties dialog! */
+		return;
 	}
 	
 	mc->property_window = gtk_dialog_new_with_buttons (_("Inbox Monitor Preferences"), 
@@ -1627,7 +1650,8 @@ mailcheck_properties (BonoboUIComponent *uic, gpointer data, const gchar *verbna
 	gtk_dialog_set_default_response (GTK_DIALOG (mc->property_window), GTK_RESPONSE_CLOSE);
 	gnome_window_icon_set_from_file (GTK_WINDOW (mc->property_window),
 					 GNOME_ICONDIR"/gnome-mailcheck.png");
-	
+	gtk_window_set_screen (GTK_WINDOW (mc->property_window),
+			       gtk_widget_get_screen (GTK_WIDGET (mc->applet)));
 	
 	notebook = gtk_notebook_new ();
 	gtk_widget_show (notebook);
@@ -1688,9 +1712,8 @@ applet_load_prefs(MailCheck *mc)
 }
 
 static void
-mailcheck_about(BonoboUIComponent *uic, gpointer data, const gchar *verbname)
+mailcheck_about(BonoboUIComponent *uic, MailCheck *mc, const gchar *verbname)
 {
-	MailCheck *mc = data;
 	GdkPixbuf *pixbuf = NULL;
 	gchar *file;
 
@@ -1707,10 +1730,10 @@ mailcheck_about(BonoboUIComponent *uic, gpointer data, const gchar *verbname)
 	};
 	const char *translator_credits = _("translator_credits");
 
-	if (mc->about != NULL)
-	{
-		gtk_widget_show_now(mc->about);
-		gdk_window_raise(mc->about->window);
+	if (mc->about) {
+		gtk_window_set_screen (GTK_WINDOW (mc->about),
+				       gtk_widget_get_screen (GTK_WIDGET (mc->applet)));
+		gtk_window_present (GTK_WINDOW (mc->about));
 		return;
 	}
 	
@@ -1727,7 +1750,8 @@ mailcheck_about(BonoboUIComponent *uic, gpointer data, const gchar *verbname)
 				     pixbuf);
 				     
 	gtk_window_set_wmclass (GTK_WINDOW (mc->about), "mailcheck", "Mailcheck");
-	
+	gtk_window_set_screen (GTK_WINDOW (mc->about),
+			       gtk_widget_get_screen (GTK_WIDGET (mc->applet)));
 
 	gnome_window_icon_set_from_file (GTK_WINDOW (mc->about),
 					 GNOME_ICONDIR"/gnome-mailcheck.png");
@@ -1759,7 +1783,7 @@ applet_change_pixel_size(PanelApplet * w, gint size, gpointer data)
 }
 
 static void
-help_callback (BonoboUIComponent *uic, gpointer data, const gchar *verbname)
+help_callback (BonoboUIComponent *uic, MailCheck *mc, const gchar *verbname)
 {
 	GError *error = NULL;
 	static GnomeProgram *applet_program = NULL;
@@ -1772,8 +1796,9 @@ help_callback (BonoboUIComponent *uic, gpointer data, const gchar *verbname)
 						      GNOME_PROGRAM_STANDARD_PROPERTIES, NULL);
 	}
 
-	gnome_help_display_desktop (applet_program, "mailcheck",
-				    "mailcheck",NULL, &error);
+	egg_screen_help_display_desktop (
+		gtk_widget_get_screen (GTK_WIDGET (mc->applet)),
+		applet_program, "mailcheck", "mailcheck",NULL, &error);
 	if (error) {
 		GtkWidget *dialog;
 		dialog = gtk_message_dialog_new (NULL,
@@ -1788,6 +1813,8 @@ help_callback (BonoboUIComponent *uic, gpointer data, const gchar *verbname)
 				  NULL);
 
 		gtk_window_set_resizable (GTK_WINDOW (dialog), FALSE);
+		gtk_window_set_screen (GTK_WINDOW (dialog),
+				       gtk_widget_get_screen (GTK_WIDGET (mc->applet)));
 		gtk_widget_show (dialog);
 		g_error_free (error);
 	}
@@ -1904,8 +1931,7 @@ fill_mailcheck_applet(PanelApplet *applet)
 					   mc);
 	
 	gtk_label_set_text (GTK_LABEL (mc->label), _("Status not updated"));
-	gtk_tooltips_set_tip (panel_tooltips, GTK_WIDGET (mc->applet),
-			      _("Status not updated"), NULL);
+	set_tooltip (GTK_WIDGET (mc->applet), _("Status not updated"));
 	set_atk_name_description (GTK_WIDGET (mc->applet), _("Mail check"), 
 			_("Mail check notifies you when new mail arrives in your mailbox"));
 	gtk_widget_show_all (GTK_WIDGET (applet));
