@@ -49,6 +49,11 @@
 #include <X11/Xutil.h>
 #include <X11/cursorfont.h>
 #include <X11/Xmu/WinUtil.h>
+
+#ifdef HAVE_X11_EXTENSIONS_SHAPE_H
+#include <X11/extensions/shape.h>
+#endif
+
 #include <libart_lgpl/art_rgb_affine.h>
 
 #ifdef HAVE_PAPER_WIDTH
@@ -73,6 +78,7 @@ static const char *home_dir;
 static char *class_name = NULL;
 static pid_t temporary_pid = 0;
 static char *temporary_file = NULL;
+static int  status;
 
 static GtkTargetEntry drag_types[] =
 	{ { "x-special/gnome-icon-list", 0, 0 },
@@ -148,9 +154,10 @@ save_to_file_internal (FILE *fp, const char *file, char **error)
 	png_init_io (png_ptr, fp);
 
 	png_set_IHDR (png_ptr, info_ptr, w, h, bpc,
-		      PNG_COLOR_TYPE_RGB, PNG_INTERLACE_NONE,
+		      PNG_COLOR_TYPE_RGB_ALPHA, PNG_INTERLACE_NONE,
 		      PNG_COMPRESSION_TYPE_BASE, PNG_FILTER_TYPE_BASE);
-	data = malloc (w * 3 * sizeof (char));
+	
+	data = malloc (w * 4 * sizeof (char));
 
 	if (data == NULL) {
 		*error = _("Insufficient memory to save the screenshot.\n"
@@ -166,11 +173,11 @@ save_to_file_internal (FILE *fp, const char *file, char **error)
 	png_write_info (png_ptr, info_ptr);
 	png_set_shift (png_ptr, &sig_bit);
 	png_set_packing (png_ptr);
-
+	
 	ptr = pixels;
 	for (y = 0; y < h; y++) {
 		for (j = 0, x = 0; x < w; x++)
-			memcpy (&(data[x*3]), &(ptr[x*3]), 3);
+			memcpy (&(data[x*4]), &(ptr[x*4]), 4);
 
 		row_ptr = (png_bytep)data;
 		png_write_rows (png_ptr, &row_ptr, 1);
@@ -221,7 +228,7 @@ nibble_on_file (const char *file)
 			 _("Unable to create the file:\n"
 			   "\"%s\"\n"
 			   "Please check your permissions of "
-			   "the parent directory"), file);
+			   "the parent directory"), file ? file : "(null)");
 		gtk_dialog_run (GTK_DIALOG (dialog));
 		gtk_widget_destroy (dialog);
 
@@ -236,14 +243,14 @@ static gboolean
 save_to_file (FILE *fp, const gchar *file, gboolean gui_errors)
 {
 	GtkWidget *dialog;
-	char *error;
+	char *error = NULL;
 
 	if (fp == NULL) {
 		fp = nibble_on_file (file);
 	}
 
 	if ( ! save_to_file_internal (fp, file, &error)) {
-		if (gui_errors) {
+		if (gui_errors && error) {
 			dialog = gtk_message_dialog_new
 				(GTK_WINDOW (toplevel),
 				 0 /* flags */,
@@ -318,7 +325,7 @@ start_temporary (void)
 	}
 
 	/* can't fork? don't dispair, do synchroniously */
-	if (temporary_pid < 0) {
+	else if (temporary_pid < 0) {
 		FILE *fp = fopen (file, "w");
 		if (fp == NULL ||
 		    ! save_to_file (fp, file, TRUE)) {
@@ -329,7 +336,9 @@ start_temporary (void)
 			return;
 		}
 		temporary_pid = 0;
-	}
+
+	} else if ( temporary_pid > 0)
+		while ((waitpid (temporary_pid, &status, 0) == -1) && errno == EINTR);
 
 	umask(old_mask);
 	temporary_file = file;
@@ -338,7 +347,6 @@ start_temporary (void)
 static gboolean
 ensure_temporary (void)
 {
-	int status;
 
 	start_temporary ();
 
@@ -348,9 +356,6 @@ ensure_temporary (void)
 	if (temporary_pid == 0)
 		return TRUE;
 
-	/* ok, gotta wait */
-	waitpid (temporary_pid, &status, 0);
-
 	temporary_pid = 0;
 
 	if (WIFEXITED (status) &&
@@ -625,16 +630,17 @@ on_preview_expose_event (GtkWidget      *drawing_area,
 			 GdkEventExpose *event,
 			 gpointer        data)
 {
-	gdk_draw_rgb_image (drawing_area->window,
-			    drawing_area->style->white_gc,
-			    event->area.x, event->area.y,
-			    event->area.width,
-			    event->area.height,
-			    GDK_RGB_DITHER_NORMAL,
-			    gdk_pixbuf_get_pixels (preview_image)
-			    + (event->area.y * gdk_pixbuf_get_rowstride (preview_image))
-			    + (event->area.x * gdk_pixbuf_get_n_channels (preview_image)),
-			    gdk_pixbuf_get_rowstride (preview_image));
+	gdk_pixbuf_render_to_drawable (preview_image,
+				       drawing_area->window,
+				       drawing_area->style->white_gc,
+				       event->area.x,
+				       event->area.y,
+				       event->area.x,
+				       event->area.y,
+				       event->area.width,
+				       event->area.height,
+				       GDK_RGB_DITHER_NORMAL,
+				       0, 0);
 }
 
 void
@@ -727,7 +733,7 @@ gimme_file (const char *filename)
 				 _("Unable to create the file:\n"
 				   "\"%s\"\n"
 				   "Please check your permissions of "
-				   "the parent directory"), filename);
+				   "the parent directory"), filename ? filename : "(null)");
 			gtk_dialog_run (GTK_DIALOG (dialog));
 			gtk_widget_destroy (dialog);
 			close (infd);
@@ -746,7 +752,7 @@ gimme_file (const char *filename)
 					 GTK_MESSAGE_ERROR,
 					 GTK_BUTTONS_OK,
 					 _("Not enough room to write file %s"),
-					 filename);
+					 filename ? filename : "(null)");
 				gtk_dialog_run (GTK_DIALOG (dialog));
 				gtk_widget_destroy (dialog);
 				return FALSE;
@@ -829,7 +835,7 @@ on_help_button_clicked (GtkWidget *widget, gpointer data)
 	GError *error = NULL;
 
         gnome_help_display_desktop (NULL, "user-guide", 
-				    "wgoeditmainmenu.xml", "goseditmainmenu-51", 
+				    "wgoseditmainmenu.xml", "goseditmainmenu-53", 
 				    &error);
 	
 	if (error) {
@@ -928,7 +934,7 @@ find_toplevel_window (int depth, Window xid, gboolean *keep_going)
 	return 0;
 }
 
-static void
+static gboolean
 take_window_shot (void)
 {
 	GdkWindow *window;
@@ -942,10 +948,17 @@ take_window_shot (void)
 	XClassHint class_hint;
 	gchar *result = NULL;
 	gboolean keep_going;
+
+#ifdef HAVE_X11_EXTENSIONS_SHAPE_H
+	XRectangle *rectangles;
+	GdkPixbuf *tmp;
+	int rectangle_count, rectangle_order, i;
+#endif
+
 	
 	disp = GDK_DISPLAY ();
 	w = GDK_ROOT_WINDOW ();
-
+	
 	XQueryPointer (disp, w, &root, &child,
 		       &unused,
 		       &unused,
@@ -958,6 +971,8 @@ take_window_shot (void)
 	} else {
 
                 window = gdk_window_foreign_new (child);
+		if (window == NULL)
+			return FALSE;
 
 		keep_going = TRUE;
 
@@ -991,13 +1006,47 @@ take_window_shot (void)
 		width = gdk_screen_width () - x_orig;
 	if (y_orig + height > gdk_screen_height ())
 		height = gdk_screen_height () - y_orig;
+
+
+#ifdef HAVE_X11_EXTENSIONS_SHAPE_H
+	tmp = gdk_pixbuf_get_from_drawable (NULL, window, NULL,
+					    x, y, 0, 0,
+					    width, height);
+
+	screenshot = gdk_pixbuf_new (GDK_COLORSPACE_RGB, TRUE, 8, width, height);
+	gdk_pixbuf_fill (screenshot, 0);
+	rectangles = XShapeGetRectangles (GDK_DISPLAY (), GDK_WINDOW_XWINDOW (window),
+					  ShapeBounding, &rectangle_count, &rectangle_order);
+
+	for (i = 0; i < rectangle_count; i++) {
+		for (y = rectangles[i].y; y < rectangles[i].y + rectangles[i].height; y++) {
+			guchar *src_pixels, *dest_pixels;
+
+			src_pixels = gdk_pixbuf_get_pixels (tmp) +
+				y * gdk_pixbuf_get_rowstride(tmp) +
+				rectangles[i].x * (gdk_pixbuf_get_has_alpha (tmp) ? 4 : 3);
+			dest_pixels = gdk_pixbuf_get_pixels (screenshot) +
+				y * gdk_pixbuf_get_rowstride (screenshot) +
+				rectangles[i].x * 4;
 			
-	screenshot = gdk_pixbuf_get_from_drawable (NULL, window,
-						   NULL,
+			for (x = rectangles[i].x; x < rectangles[i].x + rectangles[i].width; x++) {
+				*dest_pixels++ = *src_pixels ++;
+				*dest_pixels++ = *src_pixels ++;
+				*dest_pixels++ = *src_pixels ++;
+				*dest_pixels++ = 255;
+			}
+		}
+	}
+	g_object_unref (tmp);
+#else /* HAVE_X11_EXTENSIONS_SHAPE_H */
+	screenshot = gdk_pixbuf_get_from_drawable (NULL, window, NULL,
 						   x, y, 0, 0,
 						   width, height);
+#endif /* HAVE_X11_EXTENSIONS_SHAPE_H */
 
 	class_name = result;
+
+	return TRUE;
 }
 
 static void
@@ -1061,6 +1110,61 @@ drag_begin (GtkWidget *widget, GdkDragContext *context)
 	start_temporary ();
 }
 
+/* To make sure there is only one screenshot taken at a time,
+ * (Imagine key repeat for the print screen key) we hold a selection
+ * until we are done taking the screenshot
+ */
+static GtkWidget *selection_window;
+
+#define SELECTION_NAME "_GNOME_PANEL_SCREENSHOT"
+
+static gboolean
+get_lock (void)
+{
+        Atom selection_atom = gdk_x11_get_xatom_by_name (SELECTION_NAME);
+	GdkCursor *cursor;
+	gboolean result = FALSE;
+
+	XGrabServer (GDK_DISPLAY ());
+        if (XGetSelectionOwner (GDK_DISPLAY(), selection_atom) != None)
+                goto out;
+
+	selection_window = gtk_invisible_new ();
+	gtk_widget_show (selection_window);
+
+	if (!gtk_selection_owner_set (selection_window,
+				      gdk_atom_intern (SELECTION_NAME, FALSE),
+				      GDK_CURRENT_TIME)) {
+		gtk_widget_destroy (selection_window);
+		selection_window = NULL;
+		goto out;
+	}
+
+	cursor = gdk_cursor_new (GDK_WATCH);
+	gdk_pointer_grab (selection_window->window, FALSE, 0, NULL,
+			  cursor, GDK_CURRENT_TIME);
+	gdk_cursor_unref (cursor);
+
+	result = TRUE;
+
+ out:
+	XUngrabServer (GDK_DISPLAY ());
+	gdk_flush ();
+
+        return result;
+}
+
+static void
+release_lock (void)
+{
+	if (selection_window) {
+		gtk_widget_destroy (selection_window);
+		selection_window = NULL;
+	}
+
+	gdk_flush ();
+}
+
 /* main */
 int
 main (int argc, char *argv[])
@@ -1099,11 +1203,20 @@ main (int argc, char *argv[])
 	if (delay > 0) {
 		sleep (delay);	
 	}
+
+	if (!get_lock ()) {
+		g_printerr ("gnome-panel-screenshot is already running\n");
+		exit (1);
+	}
 	
-	if (window)
-		take_window_shot ();
-	else
+	if (window) {
+		if ( ! take_window_shot ()) {
+			release_lock ();
+			exit (1);
+		}
+	} else {
 		take_screen_shot ();
+	}
 
 	if (g_file_test ("gnome-panel-screenshot.glade", G_FILE_TEST_EXISTS)) {
 		xml = glade_xml_new ("gnome-panel-screenshot.glade", NULL, NULL);
@@ -1114,6 +1227,7 @@ main (int argc, char *argv[])
 	}
 	if (xml == NULL) {
 		GtkWidget *dialog;
+		release_lock ();
 		dialog = gtk_message_dialog_new
 			(NULL /* parent */,
 			 0 /* flags */,
@@ -1137,6 +1251,7 @@ main (int argc, char *argv[])
 
 	if (screenshot == NULL) {
 		GtkWidget *dialog;
+		release_lock ();
 		dialog = gtk_message_dialog_new
 			(NULL /* parent */,
 			 0 /* flags */,
@@ -1218,6 +1333,7 @@ main (int argc, char *argv[])
 			  NULL);
 
 	gtk_widget_show_now (toplevel);
+	release_lock ();
 
 	/*
 	 * Start working on the temporary file in a fork, now this is
